(************************************************************************
 Author:	Robert Rossmair
		Wagelwitz, Rodaer Str. 6
		04688 Mutzschen
		Germany

	e-mail:	Robert.Rossmair@t-online.de
	http://home.t-online.de/home/Robert.Rossmair/

 Module:	Exampl87

 Version:	17-May-1999

 Description:	Object Pascal translation of example 8.7
		from Intel SPL V4.1 Reference Manual p. 329

 Compatibility:	Delphi 2.0/Delphi 3

 Copyright  1998  Robert Rossmair

 Use, modification and distribution is allowed without limitation,
 warranty, or liability of any kind. If you use this software or portions
 of it in a commercial program, I would appreciate it if you mentioned my
 contribution in the documentation materials accompaning your work.
 ************************************************************************)
unit Exampl87;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, NSP, ZoomDisp, ComCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    ZoomDisplay: TZoomDisplay;
    StatusBar: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure ZoomDisplayPaint(Sender: TObject);
    procedure ZoomDisplayWorldPosChange(Sender: TObject; X, Y: Double);
    procedure ZoomDisplayGetVertScalePoint(Sender: TObject;
      var Value: Extended);
  private
    procedure Example87;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

const
  Len = 100;
var
  { RR: Error and ErrorMax are used by ZoomDisplay1Paint }
  Error: array[0..Len-1] of Double;
  ErrorMax: Double = 0;

procedure TForm1.Example87;
const
  TapsLen = 2;
  Step: Float = 0.6;
  Err: Double = 0;
var
  TapState: TNSPLmsTapState;
  DlyState: TNSPLmsDlyState;
  x, d, y, z: array[0..Len-1] of Double;
  h: array[0..TapsLen-1] of Double;
  n: Integer;
begin
(*
 * Generate the input and desired signals.  The input
 * signal is a sine wave with amplitude 1.0 at 0.2 Fs.
 * The desired signal is a cosine wave with amplitude
 * 2.0 at the same frequency.
 *)
  for n := 0 to Len-1 do
  begin
    x[n] := sin(2*Pi*n/10);
    d[n] := 2*cos(2*Pi*n/10);
    z[n] := 0.0;
  end;
  (* Initialize taps values to zero *)
  for n := 0 to TapsLen-1 do h[n] := 0.0;
  (* Initialize filter *)
  nspdLmslInit(NSP_LmsDefault, @h, TapsLen, Step, 0.0, 0, TapState);
  (* Initialize delay line *)
  nspdLmslInitDlyl(TapState, @z, 1, DlyState);
  (* Filter Len samples using single-rate adaptive filtering *)
  for n := 0 to Len-1 do
  begin
    y[n] := nspdLmsl(TapState, DlyState, x[n], err);
    Err := d[n] - y[n];
    Error[n] := Err;	// RR: for display only
    if Abs(Err) > ErrorMax then ErrorMax := Abs(Err);
  end;
  (* The final taps values = {2.75,-3.40} and
   * err = 0 are obtained
   *)
  ShowMessage(Format('Example 8.7 (Lms filtering) final values:'#13#10#13#10+
		     'Taps = {%.4f, %.4f} '#13#10+
                     'Err = %.4f', [h[0], h[1], Err]));
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Example87;
  ZoomDisplay.Invalidate;
end;

procedure TForm1.ZoomDisplayPaint(Sender: TObject);
var
  n: Integer;
  ScaleFactor: Double;
begin
  with ZoomDisplay do
  // plot error signal
  begin
    // draw zero amplitude line
    Pen.Style := psDot;
    Pen.Color := $808080;
    MoveTo(0, 0);
    LineTo(Len-1, 0);
    Pen.Style := psSolid;
    // red line: error signal amplitude
    Pen.Color := clRed;
    ScaleFactor := WindowTop/ErrorMax;
    MoveTo(0, Error[0]*ScaleFactor);
    for n := 1 to Len-1 do
      LineTo(n, Error[n]*ScaleFactor);
  end;
end;

procedure TForm1.ZoomDisplayWorldPosChange(Sender: TObject; X, Y: Double);
var
  n: Integer;
begin
  // n := sample number corresponding to horiz. mouse coordinate
  n := Round(X);
  // show n
  StatusBar.Panels[0].Text := Format('Sample # %d', [n]);
  // show Error[n] value
  StatusBar.Panels[1].Text := Format('Error signal amplitude: %.4f', [Error[n]]);
end;

procedure TForm1.ZoomDisplayGetVertScalePoint(Sender: TObject;
  var Value: Extended);
begin
  Value := Value * ErrorMax/ZoomDisplay.WindowTop;
end;

end.
